const regex = {
    header: /^\s+Description\s+Part #\s+.*return:/,
    lineItem: /^\s+(.*\S)    \s+(\S+)    \s+\$([0-9.-]+)\s+([A-Za-z0-9.-]+)\s+(.*)/,
    footer: /^\s+Total # of Parts Returned/,

    ro: /RO #:(.*)/,

    vehDesc: /Vehicle:   (.*)\s\s\s\s\s*Name:/,
    vehStyle: /Style:   (.*)\s\s\s\s\s*Contact:/,
    vehProd: /Prod Date:   (.*)\s\s\s\s\s*Fax:/,
    vehVin: /VIN:   (.*)\s\s\s\s\s*Phone:/,

    vendName: /Name:   (.*)$/m,
    vendContact: /Contact:   (.*)$/m,
    vendFax: /Fax:   (.*)$/m,
    vendPhone: /Phone:   (.*)Estimator:/m,
}

$( document ).ready(function() {
    $('#teng').on('click', butteng_click);
});

function showError(msg) {
    alert("Error: " + msg);
}

async function butteng_click(e) {
    let perm = await navigator.permissions.query({name:'clipboard-read'});
    if(perm.state === 'denied') {
        return showError("Page does not have permission to access clipboard.");
    }

    let text = null;
    try {
        text = await navigator.clipboard.readText();
    } catch (err) {
        return showError("Failed to read clipboard: " + err);
    }
     
    const data = processLineItems(text);
    const invNumber = getInvoice(data);
    const header = createHeader(text, invNumber);
    const table = createTable(data);

    const container = $('<div>')
        .addClass('memo')
        .append(header)
        .append(table);
    $(document.body).append(container);
}

function createHeader(text, invNumber) {
    const ro = fetchHeaderItem(text, regex.ro);
    
    const vehDesc = fetchHeaderItem(text, regex.vehDesc);
    const vehStyle = fetchHeaderItem(text, regex.vehStyle);
    const vehProd = fetchHeaderItem(text, regex.vehProd);
    const vehVin = fetchHeaderItem(text, regex.vehVin);

    const vendName = fetchHeaderItem(text, regex.vendName);
    const vendContact = fetchHeaderItem(text, regex.vendContact);
    const vendFax = fetchHeaderItem(text, regex.vendFax);
    const vendPhone = fetchHeaderItem(text, regex.vendPhone);
 
    const headline = $('<div>').addClass('headline')
        .append($('<span>').addClass('recipient').append(
          $('<span>').addClass('recipient-attn').text('ATTN: '),
          $('<span>').addClass('recipient-vendor').text(vendName),
        ))
        .append($('<span>').addClass('memorandum').text("CREDIT MEMORANDUM"));


    const headlineHr = $('<img src="black.png" >').addClass('hr');
    //const headlineHr = $('<hr >').addClass('hr');

    const headerBox = $('<div>').addClass('header-box');
    const fromBox = $('<div>').addClass('from-box').addClass('info-box');
    const fromDetails = $('<div>').addClass('box-detail');
    const toBox = $('<div>').addClass('to-box').addClass('info-box');
    const toDetails = $('<div>').addClass('box-detail');
    const vehBox = $('<div>').addClass('veh-box');

    fromBox
        .append(p('From:', 'box-header'))
        .append(fromDetails);
    fromDetails
        .append(p("North Andover Auto Body"))
        .append(p("374 Osgood St."))
        .append(p("North Andover, MA 01845"))
        .append(p("Phone: 978-683-6000"))
        .append(p("Fax: 978-688-3722"));
    toBox
        .append(p('To:', 'box-header'))
        .append(toDetails); 
    toDetails
        .append(p(vendName))
        .append(p("Contact: " + vendContact))
        .append(p("Phone: " + vendPhone))
        .append(p("Fax: " + vendFax))
        .append(p("").html('&nbsp;'));

    vehBox 
      .append($('<span>').text('Printed ' + new Date().toLocaleDateString()).addClass('veh-box-ro'))
      .append($('<span>').text('RO# ' + ro).addClass('veh-box-ro'))
      .append($('<span>').text('Original Invoice: ' + invNumber).addClass('veh-box-invoice'))
      .append($('<span>').text('VIN: ' + vehVin).addClass('veh-box-vin'));
    
    headerBox
        .append(fromBox)
        .append(toBox);
    
    return $('<div>').addClass('memo-header')
        .append(headline)
        .append(headlineHr)
        .append(headerBox)
        .append(vehBox);
}

function p(text, className){
    const result = $('<p>').text(text);
    if(className) result.addClass(className);
    return result;
}

function fetchHeaderItem(text, regex) {
    const match = text.match(regex);
    return match ? match[1].trim() : "";
}

function processLineItems(text) {
    let lines = text.split(/\r\n|\r|\n/);
    let lineNum = 0;
    let lookingForHeader = true;
    let data = [];
  
    while(lineNum < lines.length) {
        const line = lines[lineNum];
        if(lookingForHeader) {
            if(regex.header.test(line)) lookingForHeader = false;
        } else {
            if(regex.footer.test(line)) {
                lookingForHeader = true;
            } else {
                const match = line.match(regex.lineItem);
                if(match) {
                    [, desc, partNum, cost, invoice, reason] = match;
                    data.push( {desc, partNum, cost, invoice, reason} );
                } else {
                    showError("Error in line " + (lineNum + 1));
                }
            }
        }

        lineNum++;
    }
    
    return data;
}

function getInvoice(data){ 
    let invNumber = data.length ? data[0].invoice : "n/a";
    invNumber = data.reduce((result, item) => item.invoice === invNumber ? invNumber : "multiple", invNumber);
    return invNumber;
}

function createTable(data) {
    //let invNumber = data.length ? data[0].invoice : "n/a";
    //invNumber = data.reduce((result, item) => item.invoice === invNumber ? invNumber : "multiple", invNumber);
    const totalCost = data.reduce( (sum, item) => sum + parseFloat(item.cost), 0);

    let table = $('<table>').attr("cell-spacing",0).attr("cell-padding",0);
    let thead = $('<thead>');
    let theadrow = $('<tr>');
    let tbody = $('<tbody>');
    let tfootRow = $('<tr>');
    let tfoot = $('<tfoot>');
    
    thead.append(theadrow);
    table.append(thead);
    table.append(tbody);
    table.append(tfoot);
    table.addClass('teng-table');
    
    thead
        .append($('<td>').text("Description"))
        .append($('<td>').text("Part #"))
        .append($('<td>').text("Invoice"))
        .append($('<td>').text("Reason"))
        .append($('<td>').text("Cost").addClass('teng-money')); 
    
    tfoot.append(tfootRow);
    let tfootCost = $('<td>').addClass('teng-money');
    tfootCost
      .append($('<span>').html('$&nbsp;&nbsp;&nbsp;'))
      .append($('<span>').text(totalCost.toFixed(2)));
    tfootRow
      .append($('<td>').text('Total credit due:'))
      .append($('<td>'))
      .append($('<td>'))
      .append($('<td>'))
      .append(tfootCost);
    
    data.forEach(row => {
        let trow = $('<tr>');
        let tcost = $('<td>');
        tcost.append($('<span>$&nbsp;&nbsp;&nbsp;</span><span>' + row.cost + '</span>'));
        trow
            .append(cell(row.desc, 'td-desc'))
            .append(cell(row.partNum))
            .append(cell(row.invoice))
            .append(cell(row.reason, 'td-reason'))
//            .append($('<td>').text("$" + row.cost).addClass("teng-money"));
            .append(tcost.addClass("teng-money"));
        tbody.append(trow);
    });
    return table;
}

function cell(text, className) {
    const div = $('<div>').text(text);
    if(className) div.addClass(className);
    return result =  $('<td>').append(div);
}